from typing import Set, Optional
import os
from functools import partial

from PyQt5 import uic, QtCore
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap

from product.mode import Frequency, Voltage, Current, CurveState
from gui.mainwindow import MainWindow
from gui.ivcviewerborders import voltage_to_min_border, current_to_min_border
from .embeddedivcviewer import EmbeddedIvcViewer


__all__ = ["EmbeddedMainWindow"]


class EmbeddedMainWindow(MainWindow):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

        uic.loadUi(os.path.join(
            "gui", "embedded", "embeddedmainwindow.ui"), self)

        self.__ivc_viewer = EmbeddedIvcViewer(self)
        self.main_frame.layout().addWidget(self.__ivc_viewer)

        self.__frequency_to_push_button = {
            Frequency.FREQUENCY_10HZ: self.frequency_10hz_push_button,
            Frequency.FREQUENCY_100HZ: self.frequency_100hz_push_button,
            Frequency.FREQUENCY_1KHZ: self.frequency_1khz_push_button,
            Frequency.FREQUENCY_10KHZ: self.frequency_10khz_push_button
        }

        self.__voltage_to_push_button = {
            Voltage.VOLTAGE_3_3V: self.voltage_3_3v_push_button,
            Voltage.VOLTAGE_5V: self.voltage_5v_push_button,
            Voltage.VOLTAGE_12V: self.voltage_12v_push_button
        }

        self.__current_to_push_button = {
            Current.CURRENT_250UA: self.current_250ua_push_button,
            Current.CURRENT_2_5MA: self.current_2_5ma_push_button,
            Current.CURRENT_25MA: self.current_25ma_push_button
        }

        # Pull state to GUI
        self._set_score(self.score)
        self._set_enable_sound_support(self.enable_sound_support)
        self._set_enable_sound(self.enable_sound)
        self._set_frequency(self.frequency)
        self._set_frequency_enabled(self.frequency_enabled)
        self._set_voltage(self.voltage)
        self._set_voltage_enabled(self.voltage_enabled)
        self._set_current(self.current)
        self._set_current_enabled(self.current_enabled)
        self._set_reference_curve_state(self.reference_curve_state)
        self._set_test_curve_state(self.test_curve_state)
        self._set_external_freeze_state(self.external_freeze_state)
        self._set_screenshots_allowed(self.screenshots_allowed)
        self._set_debug(self.debug)
        self._set_fps(self.fps)

        # Connect GUI signals
        self.__ivc_viewer.screenshot_requested.connect(
            self.make_screenshot)

        self.sound_enabled_push_button.toggled.connect(
            self.__on_enable_sound_push_button_toggled)

        # self.eject_flash_drive_push_button.toggled.connect(
        #    self.__eject_flash_drive_push_button.toggled)

        for frequency, push_button in self.__frequency_to_push_button.items():
            push_button.toggled.connect(partial(
                self.__on_frequency_push_button_toggled, frequency))

        for voltage, push_button in self.__voltage_to_push_button.items():
            push_button.toggled.connect(partial(
                self.__on_voltage_push_button_toggled, voltage))

        for current, push_button in self.__current_to_push_button.items():
            push_button.toggled.connect(partial(
                self.__on_current_push_button_toggled, current))

        self.freeze_curve_a_push_button.toggled.connect(
            self.__on_freeze_curve_a_push_button_toggled)
        self.freeze_curve_b_push_button.toggled.connect(
            self.__on_freeze_curve_b_push_button_toggled)

        self.save_image_push_button.clicked.connect(self.make_screenshot)

        self.test_plan_tab_widget.currentChanged.connect(self.__on_test_plan_tab_changed)

        self.zp_push_button_left.clicked.connect(self.on_test_plan_zp_left)
        self.zp_push_button_right.clicked.connect(self.on_test_plan_zp_right)
        self.zp_push_button_save.clicked.connect(self.on_test_plan_zp_save)
        self.zp_push_button_new_point.clicked.connect(self.on_test_plan_zp_new_point)

        self.tp_push_button_left.clicked.connect(self.on_test_plan_tp_left)
        self.tp_push_button_right.clicked.connect(self.on_test_plan_tp_right)
        self.tp_push_button_save.clicked.connect(self.on_test_plan_tp_save)

        self.c_push_button_auto_calibration.clicked.connect(self.on_test_plan_auto_calibration)

        self.pushButton_score_threshold_minus.clicked.connect(lambda: self.on_score_threshold_button(False))
        self.pushButton_score_threshold_plus.clicked.connect(lambda: self.on_score_threshold_button(True))

    def test_plan_ui_refresh(self, index, left, right, file_found):
        for w in (self.zp_label_num, self.tp_label_num):
            w.setText(str(index + 1) if file_found else "")
        for w in (self.zp_push_button_left, self.tp_push_button_left):
            w.setEnabled(file_found and left)
        for w in (self.zp_push_button_right, self.tp_push_button_right):
            w.setEnabled(file_found and right)
        for w in (
                self.zp_push_button_save,
                self.zp_push_button_new_point,
                self.tp_push_button_save):
            w.setEnabled(file_found)

    @property
    def ivc_viewer(self):
        return self.__ivc_viewer

    def __on_frequency_push_button_toggled(
            self, frequency: Frequency, checked: bool) -> None:
        if checked:
            self.frequency = frequency

    def __on_voltage_push_button_toggled(
            self, voltage: Voltage, checked: bool) -> None:
        if checked:
            self.voltage = voltage

    def __on_current_push_button_toggled(
            self, current: Current, checked: bool) -> None:
        if checked:
            self.current = current

    @QtCore.pyqtSlot(bool)
    def __on_enable_sound_push_button_toggled(
            self, checked) -> None:
        self.enable_sound = checked

    @QtCore.pyqtSlot(bool)
    def __eject_flash_drive_push_button(
            self, checked) -> None:
        self.eject_flash_drive = checked

    @QtCore.pyqtSlot(bool)
    def __on_freeze_curve_a_push_button_toggled(
            self, checked: bool) -> None:
        self.reference_curve_state = CurveState.FROZEN\
            if checked else CurveState.NORMAL

    @QtCore.pyqtSlot(bool)
    def __on_freeze_curve_b_push_button_toggled(self, checked: bool) -> None:
        self.test_curve_state = CurveState.FROZEN\
            if checked else CurveState.NORMAL

    def _set_curves(
            self,
            reference_curve, test_curve,
            frequency: Frequency, voltage: Voltage, current: Current
    ) -> None:
        if self.reference_curve_state == CurveState.NORMAL or\
                self.test_curve_state == CurveState.NORMAL:
            self.__ivc_viewer.set_curves(
                reference_curve=reference_curve,
                test_curve=test_curve,
                voltage_min_border=voltage_to_min_border[voltage],
                current_min_border=current_to_min_border[current])
        else:
            self.__ivc_viewer.set_curves(
                reference_curve=reference_curve, test_curve=test_curve)

    def _set_score(self, score: float) -> None:
        score_str = format(score, ".2f")
        if score < self.score_threshold:
            self.score_label.setText(_score_good_format % score_str)
        else:
            self.score_label.setText(_score_bad_format % score_str)

    def _set_enable_sound_support(
            self, enable_sound_support: bool) -> None:
        self.sound_enabled_push_button.setEnabled(enable_sound_support)

    def _set_enable_sound(self, enable_sound: bool) -> None:
        self.sound_enabled_push_button.setChecked(
            Qt.Checked if enable_sound else Qt.Unchecked)

    def _set_frequency(self, frequency: Frequency) -> None:
        self.__frequency_to_push_button[frequency].setChecked(True)

    def _set_frequency_enabled(
            self, frequency_enabled: Set[Frequency]) -> None:
        for frequency in frequency_enabled:
            self.__frequency_to_push_button[frequency].setEnabled(True)
        for frequency in set(
                self.__frequency_to_push_button.keys()) - frequency_enabled:
            self.__frequency_to_push_button[frequency].setEnabled(False)

    def _set_voltage(self, voltage: Voltage) -> None:
        self.__voltage_to_push_button[voltage].setChecked(True)

    def _set_voltage_enabled(self, voltage_enabled: Set[Voltage]) -> None:
        for voltage in voltage_enabled:
            self.__voltage_to_push_button[voltage].setEnabled(True)
        for voltage in set(
                self.__voltage_to_push_button.keys()) - voltage_enabled:
            self.__voltage_to_push_button[voltage].setEnabled(False)

    def _set_current(self, current: Current) -> None:
        self.__current_to_push_button[current].setChecked(True)

    def _set_current_enabled(
            self, current_enabled: Set[Current]) -> None:
        for current in current_enabled:
            self.__current_to_push_button[current].setEnabled(True)
        for current in set(
                self.__current_to_push_button.keys()) - current_enabled:
            self.__current_to_push_button[current].setEnabled(False)

    def _set_reference_curve_state(
            self, reference_curve_state: CurveState) -> None:
        self.freeze_curve_a_push_button.setChecked(
            reference_curve_state == CurveState.FROZEN)

    def _set_test_curve_state(
            self, test_curve_state: CurveState) -> None:
        self.freeze_curve_b_push_button.setChecked(
            test_curve_state == CurveState.FROZEN)

    def _set_external_freeze_state(
            self, external_freeze_state: CurveState) -> None:
        self.freeze_curve_a_push_button.setEnabled(
            external_freeze_state == CurveState.NORMAL)
        self.freeze_curve_b_push_button.setEnabled(
            external_freeze_state == CurveState.NORMAL)
        self.freeze_curve_a_push_button.setChecked(
            self.reference_curve_state == CurveState.FROZEN)
        self.freeze_curve_b_push_button.setChecked(
            self.test_curve_state == CurveState.FROZEN)

    def _set_screenshots_allowed(self, screenshots_allowed: bool) -> None:
        self.save_image_push_button.setEnabled(screenshots_allowed)

    def _make_screenshot(self) -> Optional[QPixmap]:
        return self.grab(self.rect())

    def __on_test_plan_tab_changed(self, i):
        if i == 0:
            self.on_test_plan_tab_s()
        elif i == 1:
            self.on_test_plan_tab_zp()
        elif i == 2:
            self.on_test_plan_tab_tp()
        elif i == 3:
            self.on_test_plan_tab_c()


_score_good_format = "<html><head/><body><p><span style="\
    "\"font-size:48pt; color:#73d216;\">%s</span></p></body></html>"
_score_bad_format = "<html><head/><body><p><span style="\
    "\"font-size:48pt; color:#cc0000;\">%s</span></p></body></html>"
